import os
import unittest

from test.config import config
from smappdragon import CsvCollection
from smappdragon.tools.tweet_parser import TweetParser

class TestCsvCollection(unittest.TestCase):

    def test_iterator_returns_tweets(self):
        collection = CsvCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['csv']['valid'])
        self.assertTrue(len(list(collection.get_iterator())) > 0)

    # special test because custom logic is different on mongo
    def test_json_collection_custom_filter_filters(self):
        collectionone = CsvCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['csv']['valid'])
        full_collection_len = len(list(collectionone.get_iterator()))
        def is_tweet_a_retweet(tweet):
            if 'retweeted' in tweet and tweet['retweeted']:
                return True
            else:
                return False
        num_retweets = len(list(collectionone.set_custom_filter(is_tweet_a_retweet).get_iterator()))

        collectiontwo = CsvCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['csv']['valid'])
        def is_not_a_retweet(tweet):
            if 'retweeted' in tweet and tweet['retweeted']:
                return False
            else:
                return True
        num_non_retweets = len(list(collectiontwo.set_custom_filter(is_not_a_retweet).get_iterator()))

        #the numbes of retweets and non retweets should add up to the whole collection
        self.assertEqual(num_retweets + num_non_retweets, full_collection_len)
        
    def test_strip_tweets_keeps_fields(self):
        tweet_parser = TweetParser()
        collection = CsvCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['csv']['valid'])
        self.maxDiff = None
        it = collection.strip_tweets(['source', 'text', 'id_str']).get_iterator()
        def tweets_have_right_keys(iterator, fields):
            for tweet in iterator:
                keys = [key for key,value in tweet_parser.flatten_dict(tweet)]
                for elem in fields:
                    if elem not in keys:
                        return False
            return True     
        self.assertTrue(tweets_have_right_keys(it, [['source'], ['text'], ['id_str']]))


if __name__ == '__main__':
    unittest.main()



import csv
import abc
import json
import sqlite3
import operator

from bson import BSON, json_util
from smappdragon.tools.tweet_parser import TweetParser

class BaseCollection(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def __init__(self):
        self.limit = 0
        self.filter = {}
        self.keep_fields = []
        self.should_strip = False
        self.custom_filters = []

    '''
        returns an iterator that
        can iterate through the tweets
        in a collection
    '''
    @abc.abstractmethod
    def get_iterator(self):
        pass

    '''
        sets the fields to keep when
        the user calls strip tweets
    '''
    def strip_tweets(self, keep_fields):
        self.keep_fields = keep_fields
        self.should_strip = True
        return self

    '''
        returns the modified collection
        object with a limit on how many tweets
        it will ever output or query
    '''
    def set_limit(self, limit):
        self.limit = limit
        return self

    '''
        sets the filters you'd
        like to apply to the query
        follows mongdb query syntax
    '''
    def set_filter(self, query_filter):
        self.filter = query_filter
        return self

    '''
        takes a function as an input
        and appends it to the list of
        custom filters that need to be passed
    '''
    def set_custom_filter(self, func):
        self.custom_filters.append(func)
        return self

    '''
        takes a function as an input
        and appends it to the list of
        custom filters that need to be passed
    '''
    def set_custom_filter_list(self, functions_list):
        self.custom_filters.extend(functions_list)
        return self

    '''
        dumps the contents of a collection 
        to a bson file, this is a binary format
    '''
    def dump_to_bson(self, output_bson):
        filehandle = open(output_bson, 'ab+')

        for tweet in self.get_iterator():
            filehandle.write(BSON.encode(tweet))
        filehandle.close()

    '''
        dumps the contents of a collection
        to a json file, a json object on
        each line
    '''
    def dump_to_json(self, output_json):
        filehandle = open(output_json, 'a')

        for tweet in self.get_iterator():
            filehandle.write(json_util.dumps(tweet)+'\n')
        filehandle.close()

    '''
        dumps the contents of a collection 
        to csv format with columns specified
        by input_fields
    '''
    def dump_to_csv(self, output_csv, input_fields, write_header=True):
        filehandle = open(output_csv, 'a', encoding='utf-8')
        writer = csv.writer(filehandle)
        if write_header:
            writer.writerow(input_fields)
        tweet_parser = TweetParser()

        for tweet in self.get_iterator():
            ret = tweet_parser.parse_columns_from_tweet(tweet,input_fields)
            ret_values = [col_val[1] for col_val in ret]
            writer.writerow(ret_values)
        filehandle.close()

    '''
        dumps the contents of a collection 
        to an sqlite database with columns
        specified by input_fields
    '''
    def dump_to_sqlite_db(self, output_db, input_fields):
        def replace_none(s):
            if s is None:
                return 'NULL'
            return s

        tweet_parser = TweetParser()
        column_str = ','.join([column for column in input_fields]).replace('.','__')
        question_marks = ','.join(['?' for column in input_fields])

        con = sqlite3.connect(output_db)
        cur = con.cursor()
        cur.execute("CREATE TABLE IF NOT EXISTS data ({});".format(column_str))

        insert_list = []
        for count,tweet in enumerate(self.get_iterator()):
            ret = tweet_parser.parse_columns_from_tweet(tweet, input_fields)
            row = [replace_none(col_val[1]) for col_val in ret]
            insert_list.append(tuple(row))
            if (count % 10000) == 0:
                cur.executemany("INSERT INTO data ({}) VALUES ({});".format(column_str, question_marks), insert_list)
                con.commit()
                insert_list = []
        if count < 10000:
            cur.executemany("INSERT INTO data ({}) VALUES ({});".format(column_str, question_marks), insert_list)
            con.commit()
        con.close()



import os

from bson import json_util
from smappdragon.tools.tweet_parser import TweetParser
from smappdragon.collection.base_collection import BaseCollection

class JsonCollection(BaseCollection):
	'''
		method that tells us how to
		create the JsonCollection object
	'''
	def __init__(self, filepath):
		BaseCollection.__init__(self)
		self.filepath = filepath
		if not os.path.isfile(filepath):
			raise IOError(filepath, 'JsonCollection could not find your file, it\'s mispelled or doesn\'t exist.')

	'''
		method that creates a cursor
		and yields all tweets in a particular collection
		expects a json object on each line
		no spaghetti string
	'''
	def get_iterator(self):
		tweet_parser = TweetParser()
		json_handle = open(self.filepath, 'r', encoding='utf-8')
		for count, tweet in enumerate(json_handle):
			try:
				tweet = json_util.loads(tweet)
			except ValueError:
				continue
			if self.limit != 0 and self.limit <= count:
				return
			elif tweet_parser.tweet_passes_filter(self.filter, tweet) \
			and tweet_parser.tweet_passes_custom_filter_list(self.custom_filters, tweet):
				if self.should_strip:
					yield tweet_parser.strip_tweet(self.keep_fields, tweet) 
				else: 
					yield tweet
		json_handle.close()



import sys
from setuptools import setup

setup(name='smappdragon',
	packages=['smappdragon', 'smappdragon.collection', 'smappdragon.tools'],
	version='0.0.41',
	description='smappdragon is a set of tools for working with twitter data',
	author='yvan',
	author_email='yns207@nyu.edu',
	url='https://github.com/SMAPPNYU/smappdragon',
	keywords='twitter parse smappdragon',
	license='MIT',
	install_requires=[
	  'pymongo>=3.2.1'
	]
)


import unittest
from smappdragon import TweetParser

class TestTweetParser(unittest.TestCase):
	tweet_parser = TweetParser()

	def test_contains_entity_empty(self):
		self.assertFalse( \
						self.tweet_parser.contains_entity( \
							'user_mentions', \
							{'blah':1, 'entities':{'user_mentions':[]}} \
						) \
		)

	def test_contains_entity(self):
		tweet_object = { \
			'blah':1, \
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	} \
		}
		self.assertTrue(self.tweet_parser.contains_entity('user_mentions', tweet_object))

	def test_get_entity_empty(self):
		self.assertEqual([], self.tweet_parser.get_entity('urls', {'blah':1, 'user_mentions':[]}))

	def test_get_entity(self):
		url_entity_objects = [ \
			{ \
		      "url": "https://t.co/XdXRudPXH5", \
		      "expanded_url": "https://blog.twitter.com/2013/rich-photo-experience-now-in-embedded-tweets-3", \
		      "display_url": "blog.twitter.com/2013/rich-phot", \
		      "indices": [80, 103] \
		    }, \
			{ \
		      "url": "https://t.co/XdXRudPXH4", \
		      "expanded_url": "https://blog.twitter.com/2013/rich-photo-experience-now-in-embedded-tweets-3", \
		      "display_url": "blog.twitter.com/2013/rich-deio", \
		      "indices": [80, 103] \
		    } \
		]
		self.assertEqual(url_entity_objects, self.tweet_parser.get_entity('urls', {'blah':1, 'entities':{'urls':url_entity_objects}}))

	def test_get_entity_field(self):
		url_entity_object = { \
		      "url": "https://t.co/XdXRudPXH5", \
		      "expanded_url": "https://blog.twitter.com/2013/rich-photo-experience-now-in-embedded-tweets-3", \
		      "display_url": "blog.twitter.com/2013/rich-phot", \
		      "indices": [80, 103] \
		    }
		self.assertEqual('blog.twitter.com/2013/rich-phot', self.tweet_parser.get_entity_field('display_url', url_entity_object))

	def test_flatten_dict_flattens(self):
		dict_to_flatten = {'a':'b', 'c':{'d':'e', 'f':{'g':'h', 'i':'j'}}}
		actual_flat_dict = [(['a'], 'b'), (['c', 'd'], 'e'), (['c', 'f', 'i'], 'j'), (['c', 'f', 'g'], 'h')]
		dict_was_flattened = True
		for entry in self.tweet_parser.flatten_dict(dict_to_flatten):
			if entry not in actual_flat_dict:
				dict_was_flattened = False
		self.assertTrue(dict_was_flattened)

	def tests_tweet_passes_empty_filter(self):
		tweet_to_test = {'a':'b', 'c':{'d':'e', 'f':{'g':'h', 'i':'j'}}}
		self.assertTrue(self.tweet_parser.tweet_passes_filter({}, tweet_to_test))

	def test_tweet_passes_filter_of_itself(self):
		tweet_to_test = {'a':'b', 'c':{'d':'e', 'f':{'g':'h', 'i':'j'}}}
		filter_obj = {'a':'b', 'c':{'d':'e', 'f':{'g':'h', 'i':'j'}}}
		self.assertTrue(self.tweet_parser.tweet_passes_filter(filter_obj, tweet_to_test))

	def test_tweet_passes_filter(self):
		tweet_to_test = {'a':'b', 'c':{'d':'e', 'f':{'g':'h', 'i':'j'}}}
		filter_obj = {'a':'b'}
		self.assertTrue(self.tweet_parser.tweet_passes_filter(filter_obj, tweet_to_test))

	def tests_tweet_fails_on_bad_filter(self):
		tweet_to_test = {'a':'b', 'c':{'d':'e', 'f':{'g':'h', 'i':'j'}}}
		filter_obj = {'t':'m'}
		self.assertFalse(self.tweet_parser.tweet_passes_filter(filter_obj, tweet_to_test))

	def test_tweet_passes_good_custom_filter(self):
		tweet_object = { \
			'blah':1, \
			'retweeted':True,
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	} \
		}
		def is_tweet_a_retweet(tweet):
			if 'retweeted' in tweet and tweet['retweeted']:
				return True
			else:
				return False
		self.assertTrue(self.tweet_parser.tweet_passes_custom_filter(is_tweet_a_retweet, tweet_object))

	def test_tweet_does_not_pass_incorrect_custom_filter(self):
		tweet_object = { \
			'blah':1, \
			'retweeted':False,
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	} \
		}
		def is_tweet_a_retweet(tweet):
			if 'retweeted' in tweet and tweet['retweeted']:
				return True
			else:
				return False
		self.assertFalse(self.tweet_parser.tweet_passes_custom_filter(is_tweet_a_retweet, tweet_object))

	def test_tweet_passes_good_custom_filter_list(self):
		tweet_object = { \
			'blah':1, \
			'retweeted':True,
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	} \
		}
		def is_tweet_a_retweet(tweet):
			if 'retweeted' in tweet and tweet['retweeted']:
				return True
			else:
				return False
		def tweet_has_blah(tweet):
			if 'blah' in tweet and tweet['blah'] == 1:
				return True
			else:
				return False
		self.assertTrue(self.tweet_parser.tweet_passes_custom_filter_list([is_tweet_a_retweet, tweet_has_blah], tweet_object))

	def test_tweet_does_not_pass_incorrect_custom_filter_list(self):
		tweet_object = { \
			'blah':1, \
			'retweeted':False,
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	} \
		}
		def is_tweet_a_retweet(tweet):
			if 'retweeted' in tweet and tweet['retweeted']:
				return True
			else:
				return False
		def tweet_has_blah(tweet):
			if 'blah' in tweet and tweet['blah'] == 1:
				return True
			else:
				return False
		self.assertFalse(self.tweet_parser.tweet_passes_custom_filter_list([is_tweet_a_retweet, tweet_has_blah], tweet_object))

	def test_strip_tweets_totally_strips_tweet(self):
		tweet_object = { \
			'blah':1, \
			'retweeted':False,
			'created_at': 'time',
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	} \
		}

		self.assertEqual(self.tweet_parser.strip_tweet([], tweet_object), {})

	def test_strip_tweets_keeps_fields(self):
		tweet_object = { \
			'blah':1, \
			'retweeted':False,
			'created_at': 'time',
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	}, \
		  	"user" : {
				"follow_request_sent" : None,
				"profile_use_background_image" : True,
				"default_profile_image" : False,
				"geo_enabled" : False,
				"verified" : False,
				"profile_image_url_https" : "https://pbs.twimg.com/profile_images/496694241536397313/zQY6Kebr_normal.jpeg",
				"profile_sidebar_fill_color" : "DDEEF6",
				"id" : 379851447,
				"profile_text_color" : "333333",
				"followers_count" : 3159,
				"profile_sidebar_border_color" : "C0DEED",
				"id_str" : "379851447",
				"profile_background_color" : "C0DEED",
				"listed_count" : 401,
				"utc_offset" : 0,
				"statuses_count" : 477638,
				"description" : "#gaza #palestine #israel #BDS MAD EVIL ISRAEL MURDERS BABIES CIVILIANS to STEAL PALESTINIAN LAND RESOURCES with USA UK HELP. To stop my tweets, BLOCK or MUTE me",
				"friends_count" : 2019,
				"profile_link_color" : "0084B4",
				"profile_image_url" : "http://pbs.twimg.com/profile_images/496694241536397313/zQY6Kebr_normal.jpeg",
				"following" : None,
				"time_zone" : "London",
				"profile_background_image_url_https" : "https://abs.twimg.com/images/themes/theme1/bg.png",
				"profile_banner_url" : "https://pbs.twimg.com/profile_banners/379851447/1416509762",
				"profile_background_image_url" : "http://abs.twimg.com/images/themes/theme1/bg.png",
				"name" : "ISRAEL BOMBS BABIES",
				"lang" : "en",
				"profile_background_tile" : False,
				"favourites_count" : 15917,
				"screen_name" : "Col_Connaughton",
				"notifications" : None,
				"url" : None,
				"created_at" : "Sun Sep 25 17:29:09 +0000 2011",
				"contributors_enabled" : False,
				"location" : "London UK",
				"protected" : False,
				"default_profile" : True,
				"is_translator" : False
			}
		}
		stripped_obj = {
			'blah':1,
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	}, \
		  	"user": {
		  		"profile_image_url_https" : "https://pbs.twimg.com/profile_images/496694241536397313/zQY6Kebr_normal.jpeg"
		  	}
		}

		self.maxDiff = None
		self.assertEqual(self.tweet_parser.strip_tweet(['blah', 'entities.user_mentions', 'user.profile_image_url_https'], tweet_object), stripped_obj)

	def test_parses_columns_from_tweet(self):
		tweet_object = { \
			'blah':1, \
			'retweeted':False,
			'created_at': 'time',
			'entities':{ \
				'user_mentions':[ \
					{ \
				      "screen_name": "TwitterEng", \
				      "name": "Twitter Engineering", \
				      "id": 6844292, \
				      "id_str": "6844292", \
				      "indices": [81, 92] \
				    }, { \
				      "screen_name": "TwitterOSS", \
				      "name": "Twitter Open Source", \
				      "id": 376825877, \
				      "id_str": "376825877", \
				      "indices": [121, 132] \
				    } \
				] \
		  	}, \
		  	"user" : {
				"follow_request_sent" : None,
				"profile_use_background_image" : True,
				"default_profile_image" : False,
				"geo_enabled" : False,
				"verified" : False,
				"profile_image_url_https" : "https://pbs.twimg.com/profile_images/496694241536397313/zQY6Kebr_normal.jpeg",
				"profile_sidebar_fill_color" : "DDEEF6",
				"id" : 379851447,
				"profile_text_color" : "333333",
				"followers_count" : 3159,
				"profile_sidebar_border_color" : "C0DEED",
				"id_str" : "379851447",
				"profile_background_color" : "C0DEED",
				"listed_count" : 401,
				"utc_offset" : 0,
				"statuses_count" : 477638,
				"description" : "#gaza #palestine #israel #BDS MAD EVIL ISRAEL MURDERS BABIES CIVILIANS to STEAL PALESTINIAN LAND RESOURCES with USA UK HELP. To stop my tweets, BLOCK or MUTE me",
				"friends_count" : 2019,
				"profile_link_color" : "0084B4",
				"profile_image_url" : "http://pbs.twimg.com/profile_images/496694241536397313/zQY6Kebr_normal.jpeg",
				"following" : None,
				"time_zone" : "London",
				"profile_background_image_url_https" : "https://abs.twimg.com/images/themes/theme1/bg.png",
				"profile_banner_url" : "https://pbs.twimg.com/profile_banners/379851447/1416509762",
				"profile_background_image_url" : "http://abs.twimg.com/images/themes/theme1/bg.png",
				"name" : "ISRAEL BOMBS BABIES",
				"lang" : "en",
				"profile_background_tile" : False,
				"favourites_count" : 15917,
				"screen_name" : "Col_Connaughton",
				"notifications" : None,
				"url" : None,
				"created_at" : "Sun Sep 25 17:29:09 +0000 2011",
				"contributors_enabled" : False,
				"location" : "London UK",
				"protected" : False,
				"default_profile" : True,
				"is_translator" : False
			}
		}
		ret = self.tweet_parser.parse_columns_from_tweet(tweet_object, ['blah', 'entities.user_mentions.0.id_str', 'entities.user_mentions.1.id_str', 'user.wrong_ass_index', 'user.profile_sidebar_fill_color', 'user.profile_background_tile'])
		ret_values = [col_val[1] for col_val in ret]
		self.assertEqual(ret_values, [1, '6844292', '376825877', None, 'DDEEF6', False])


if __name__ == '__main__':
	unittest.main()

'''
read about twitter entities here:
https://dev.twitter.com/overview/api/entities-in-twitter-objects
'''
